@@ -40,11 +40,23 @@ var proxiedFunctionInvokers = {};
 // But we can avoid emitting all the others in many cases.
 var NEED_ALL_ASM2WASM_IMPORTS = BINARYEN_TRAP_MODE == 'js';
 
-// used internally. set when there is a main() function.
-// also set when in a linkable module, as the main() function might
+// Used internally. set when there is a main() function.
+// Also set when in a linkable module, as the main() function might
 // arrive from a dynamically-linked library, and not necessarily
 // the current compilation unit.
-var HAS_MAIN = ('_main' in IMPLEMENTED_FUNCTIONS) || MAIN_MODULE || SIDE_MODULE;
+// Also set for STANDALONE_WASM since the _start function is needed to call
+// static ctors, even if there is no user main.
+var HAS_MAIN = ('_main' in IMPLEMENTED_FUNCTIONS) || MAIN_MODULE || SIDE_MODULE || STANDALONE_WASM;
+
+// Mangles the given C/JS side function name to assembly level function name (adds an underscore)
+function mangleCSymbolName(f) {
+  return f[0] == '$' ? f.substr(1) : '_' + f;
+}
+
+// Reverses C/JS name mangling: _foo -> foo, and foo -> $foo.
+function demangleCSymbolName(f) {
+  return f[0] == '_' ? f.substr(1) : '$' + f;
+}
 
 // JSifier
 function JSify(data, functionsOnly) {
@@ -72,7 +84,7 @@ function JSify(data, functionsOnly) {
     }
     libFuncsToInclude.forEach(function(ident) {
       data.functionStubs.push({
-        ident: '_' + ident
+        ident: mangleCSymbolName(ident)
       });
     });
   }
@@ -98,25 +110,26 @@ function JSify(data, functionsOnly) {
                   '}).apply(this, arguments); if (runtimeDebug && typeof ret !== "undefined") err("  [     return:" + prettyPrint(ret)); return ret; \n}\n';
         });
       }
-      if (WASM_BACKEND && ASYNCIFY && ASSERTIONS && ASYNCIFY_IMPORTS.indexOf(ident) < 0) {
-        // Only functions in the list of known relevant imports are allowed to change the state.
-        snippet = modifyFunction(snippet, function(name, args, body) {
-          return 'function ' + name + '(' + args + ') {\n' +
-                 '  var originalAsyncifyState = Asyncify.state;\n' +
-                 '  try {\n' +
-                 body + '\n' +
-                 '  } finally {\n' +
-                 '    if (Asyncify.state !== originalAsyncifyState) throw "import ' + ident + ' was not in ASYNCIFY_IMPORTS, but changed the state";\n' +
-                 '  }\n' +
-                 '}\n';
-        });
-      }
     }
     return snippet;
   }
 
   // functionStub
   function functionStubHandler(item) {
+    // In LLVM, exceptions generate a set of functions of form __cxa_find_matching_catch_1(), __cxa_find_matching_catch_2(), etc.
+    // where the number specifies the number of arguments. In Emscripten, route all these to a single function '__cxa_find_matching_catch'
+    // that variadically processes all of these functions using JS 'arguments' object.
+    if (item.ident.startsWith('___cxa_find_matching_catch_')) {
+      if (DISABLE_EXCEPTION_THROWING) {
+        error('DISABLE_EXCEPTION_THROWING was set (likely due to -fno-exceptions), which means no C++ exception throwing support code is linked in, but exception catching code appears. Either do not set DISABLE_EXCEPTION_THROWING (if you do want exception throwing) or compile all source files with -fno-except (so that no exceptions support code is required); also make sure DISABLE_EXCEPTION_CATCHING is set to the right value - if you want exceptions, it should be off, and vice versa.');
+        return;
+      }
+      var num = +item.ident.split('_').slice(-1)[0];
+      addCxaCatch(num);
+      // Continue, with the code below emitting the proper JavaScript based on
+      // what we just added to the library.
+    }
+
     function addFromLibrary(ident) {
       if (ident in addedLibraryItems) return '';
       addedLibraryItems[ident] = true;
@@ -129,12 +142,7 @@ function JSify(data, functionsOnly) {
         return '';
       }
 
-      // $ident's are special, we do not prefix them with a '_'.
-      if (ident[0] === '$') {
-        var finalName = ident.substr(1);
-      } else {
-        var finalName = '_' + ident;
-      }
+      var finalName = mangleCSymbolName(ident);
 
       // if the function was implemented in compiled code, we just need to export it so we can reach it from JS
       if (finalName in IMPLEMENTED_FUNCTIONS) {
@@ -153,16 +161,12 @@ function JSify(data, functionsOnly) {
         usedExternPrimitives[ident] = 1;
         return;
       } else if ((!LibraryManager.library.hasOwnProperty(ident) && !LibraryManager.library.hasOwnProperty(ident + '__inline')) || SIDE_MODULE) {
-        if (!(finalName in IMPLEMENTED_FUNCTIONS)) {
-          if (VERBOSE || ident.substr(0, 11) !== 'emscripten_') { // avoid warning on emscripten_* functions which are for internal usage anyhow
-            if (!LINKABLE) {
-              if (ERROR_ON_UNDEFINED_SYMBOLS) {
-                error('undefined symbol: ' + ident);
-                warnOnce('To disable errors for undefined symbols use `-s ERROR_ON_UNDEFINED_SYMBOLS=0`')
-              } else if (VERBOSE || WARN_ON_UNDEFINED_SYMBOLS) {
-                warn('undefined symbol: ' + ident);
-              }
-            }
+        if (!(finalName in IMPLEMENTED_FUNCTIONS) && !LINKABLE) {
+          if (ERROR_ON_UNDEFINED_SYMBOLS) {
+            error('undefined symbol: ' + ident);
+            warnOnce('To disable errors for undefined symbols use `-s ERROR_ON_UNDEFINED_SYMBOLS=0`')
+          } else if (VERBOSE || WARN_ON_UNDEFINED_SYMBOLS) {
+            warn('undefined symbol: ' + ident);
           }
         }
         if (!RELOCATABLE) {
@@ -175,7 +179,7 @@ function JSify(data, functionsOnly) {
             realIdent = realIdent.substr(2);
           }
 
-          var target = (SIDE_MODULE ? 'parent' : '') + "Module['_" + realIdent + "']";
+          var target = (SIDE_MODULE ? 'parent' : '') + "Module['" + mangleCSymbolName(realIdent) + "']";
           var assertion = '';
           if (ASSERTIONS) {
             var what = 'function';
@@ -211,18 +215,20 @@ function JSify(data, functionsOnly) {
       var isFunction = false;
 
       if (typeof snippet === 'string') {
-        var target = LibraryManager.library[snippet];
-        if (target) {
-          // Redirection for aliases. We include the parent, and at runtime make ourselves equal to it.
-          // This avoid having duplicate functions with identical content.
-          redirectedIdent = snippet;
-          deps.push(snippet);
-          snippet = '_' + snippet;
-        }
-        // In asm, we need to know about library functions. If there is a target, though, then no
-        // need to consider this a library function - we will call directly to it anyhow
-        if (!redirectedIdent && (typeof target == 'function' || /Math_\w+/.exec(snippet))) {
-          Functions.libraryFunctions[finalName] = 1;
+        if (snippet[0] != '=') {
+          var target = LibraryManager.library[snippet];
+          if (target) {
+            // Redirection for aliases. We include the parent, and at runtime make ourselves equal to it.
+            // This avoid having duplicate functions with identical content.
+            redirectedIdent = snippet;
+            deps.push(snippet);
+            snippet = mangleCSymbolName(snippet);
+          }
+          // In asm, we need to know about library functions. If there is a target, though, then no
+          // need to consider this a library function - we will call directly to it anyhow
+          if (!redirectedIdent && (typeof target == 'function' || /Math_\w+/.exec(snippet))) {
+            Functions.libraryFunctions[finalName] = 1;
+          }
         }
       } else if (typeof snippet === 'object') {
         snippet = stringifyWithFunctions(snippet);
@@ -282,9 +288,18 @@ function JSify(data, functionsOnly) {
           contentText = snippet; // Regular JS function that will be executed in the context of the calling thread.
         }
       } else if (typeof snippet === 'string' && snippet.indexOf(';') == 0) {
+        // In JS libraries
+        //   foo: ';[code here verbatim]'
+        //  emits
+        //   'var foo;[code here verbatim];'
         contentText = 'var ' + finalName + snippet;
         if (snippet[snippet.length-1] != ';' && snippet[snippet.length-1] != '}') contentText += ';';
       } else {
+        // In JS libraries
+        //   foo: '=[value]'
+        //  emits
+        //   'var foo = [value];'
+        if (typeof snippet === 'string' && snippet[0] == '=') snippet = snippet.substr(1);
         contentText = 'var ' + finalName + '=' + snippet + ';';
       }
       var sig = LibraryManager.library[ident + '__sig'];
@@ -307,7 +322,7 @@ function JSify(data, functionsOnly) {
         // We set EXPORTED_FUNCTIONS here to tell emscripten.py to do that.
         deps.forEach(function(dep) {
           if (LibraryManager.library[dep + '__asm']) {
-            EXPORTED_FUNCTIONS['_' + dep] = 0;
+            EXPORTED_FUNCTIONS[mangleCSymbolName(dep)] = 0;
           }
         });
       }
@@ -315,7 +330,7 @@ function JSify(data, functionsOnly) {
     }
 
     itemsDict.functionStub.push(item);
-    var shortident = item.ident.substr(1);
+    var shortident = demangleCSymbolName(item.ident);
     // If this is not linkable, anything not in the library is definitely missing
     if (item.ident in DEAD_FUNCTIONS) {
       if (LibraryManager.library[shortident + '__asm']) {
@@ -408,12 +423,14 @@ function JSify(data, functionsOnly) {
 
       if (!SIDE_MODULE && !WASM_BACKEND) {
         if (USE_PTHREADS) {
-          print('var tempDoublePtr;');
-          print('if (!ENVIRONMENT_IS_PTHREAD) tempDoublePtr = ' + makeStaticAlloc(12) + ';');
+          print('// Pthreads fill their tempDoublePtr memory area into the pthread stack when the thread is run.')
+          // Main thread still statically allocate tempDoublePtr - although it could theorerically also use its stack
+          // (that might allow removing the whole tempDoublePtr variable altogether from the codebase? but would need
+          // more refactoring)
+          print('var tempDoublePtr = ENVIRONMENT_IS_PTHREAD ? 0 : ' + makeStaticAlloc(8) + ';');
         } else {
-          print('var tempDoublePtr = ' + makeStaticAlloc(8) + '');
+          print('var tempDoublePtr = ' + makeStaticAlloc(8) + ';');
         }
-        if (ASSERTIONS) print('assert(tempDoublePtr % 8 == 0);');
         print('\nfunction copyTempFloat(ptr) { // functions, because inlining this code increases code size too much');
         print('  HEAP8[tempDoublePtr] = HEAP8[ptr];');
         print('  HEAP8[tempDoublePtr+1] = HEAP8[ptr+1];');
