@@ -572,10 +572,13 @@ mergeInto(LibraryManager.library, {
     exportCallStack: [],
     afterUnwind: null,
     asyncFinalizers: [], // functions to run when *all* asynchronicity is done
+    sleepCallbacks: [], // functions to call every time we sleep
 
 #if ASSERTIONS
     instrumentWasmImports: function(imports) {
-      var ASYNCIFY_IMPORTS = {{{ JSON.stringify(ASYNCIFY_IMPORTS) }}};
+      var ASYNCIFY_IMPORTS = {{{ JSON.stringify(ASYNCIFY_IMPORTS) }}}.map(function(x) {
+        return x.split('.')[1];
+      });
       for (var x in imports) {
         (function(x) {
           var original = imports[x];
@@ -746,6 +749,10 @@ mergeInto(LibraryManager.library, {
         runAndAbortIfError(Module['_asyncify_stop_rewind']);
         Asyncify.freeData(Asyncify.currData);
         Asyncify.currData = null;
+        // Call all sleep callbacks now that the sleep-resume is all done.
+        Asyncify.sleepCallbacks.forEach(function(func) {
+          func();
+        });
       } else {
         abort('invalid state: ' + Asyncify.state);
       }
@@ -810,6 +817,17 @@ mergeInto(LibraryManager.library, {
     });
   },
 
+  emscripten_lazy_load_code: function() {
+    Asyncify.handleSleep(function(wakeUp) {
+      // Update the expected wasm binary file to be the lazy one.
+      wasmBinaryFile += '.lazy.wasm';
+      // Add a callback for when all run dependencies are fulfilled, which happens when async wasm loading is done.
+      dependenciesFulfilled = wakeUp;
+      // Load the new wasm.
+      asm = createWasm();
+    });
+  },
+
 #else // ASYNCIFY
   emscripten_sleep: function() {
     throw 'Please compile your program with async support in order to use asynchronous operations like emscripten_sleep';
